#include <stdio.h>
#include <tusb.h>
#include "lib/st7735.h"
#include "lib/fonts.h"
#include "hardware/pwm.h"
#include "hardware/gpio.h"
#include "hardware/clocks.h"
#include "hardware/vreg.h"
#include "arducam/arducam.h"
#include "pico/stdlib.h"
#include "pico/multicore.h"
#include "pico/time.h"
#include "lib/notes.h"
#include "lib/music.h"

#define FLAG_VALUE 123
#define PWM_PIN 12		//music Pinout

uint16_t foregroundColor = (ST7735_YELLOW|ST7735_RED);  // 定义前景色
uint16_t backgroundColor = ST7735_WHITE;   // 定义背景色
uint8_t image_buf[324*324];
uint8_t displayBuf[240*135*2];
uint8_t header[2] = {0x55,0xAA};

const uint BUTTON_PIN1 = 18;   //Key1
const uint BUTTON_PIN2 = 19;   //Key2
const uint LED_PIN1 = 2;       //led1
const uint LED_PIN2 = 3;       //led2

extern note_struct HappyBirday[];
extern note_struct HarryPotter[];
extern note_struct All_Music[];
extern note_struct All_Music[];
extern note_struct Power_On_Prompt[];
extern FontDef Font_16x26;
//music

void core1_entry() {
  multicore_fifo_push_blocking(FLAG_VALUE);

  uint32_t g = multicore_fifo_pop_blocking();

  if (g != FLAG_VALUE)
    printf("Hmm, that's not right on core 1!\n");
  else
    printf("It's all gone well on core 1!\n");

	//gpio_init(PIN_LED);
	//gpio_set_dir(PIN_LED, GPIO_OUT);
	ST7735_Init();
	ST7735_InvertColors(1);  //颜色反转
  //ST7789_Rotate(1);
 
	//GJW July 13,2023
	ST7789_DrawImage(52,40,135,240, gImage_Sportpear);
  //ST7735_FillScreen(ST7735_RED);
  //ST7735_WriteString(50, 50, "A", Font_16x26, foregroundColor, backgroundColor);
  //ST7735_WriteChar(100, 200, 'A', Font_16x26, foregroundColor, backgroundColor);

	struct arducam_config config;
	config.sccb = i2c0;
	config.sccb_mode = I2C_MODE_16_8;
	config.sensor_address = 0x24;
	config.pin_sioc = PIN_CAM_SIOC;
	config.pin_siod = PIN_CAM_SIOD;
	config.pin_resetb = PIN_CAM_RESETB;
	config.pin_xclk = PIN_CAM_XCLK;
	config.pin_vsync = PIN_CAM_VSYNC;
	config.pin_y2_pio_base = PIN_CAM_Y2_PIO_BASE;

	config.pio = pio0;
	config.pio_sm = 0;

	config.dma_channel = 0;
	config.image_buf = image_buf;
	config.image_buf_size = sizeof(image_buf);
  //ST7789_Rotate(0);
	arducam_init(&config);
	
  /*
  纵向
  while (true) {
	  gpio_put(PIN_LED, !gpio_get(PIN_LED));
	  arducam_capture_frame(&config);

	  uint16_t index = 0;
	  for (int y = 0; y < 240; y++) {
	    for (int x = 0; x < 135; x++) {
		// for (int y = 0; y < 240; y++) {
	    // for (int x = 0; x < 135; x++) {
              //uint8_t c = image_buf[(2+320-2*y)*324+(2+40+2*x)];
			  //uint8_t c = image_buf[(2+480-2*y)*324+(2+40+2*x)];
			        uint16_t c = image_buf[(x)*324+(y)];
              uint16_t imageRGB   = ST7735_COLOR565(c, c, c);
              displayBuf[index++] = (uint16_t)(imageRGB >> 8) & 0xFF;
              displayBuf[index++] = (uint16_t)(imageRGB)&0xFF;
            }
	  }
	  ST7789_DrawImage(52,40,135,240,displayBuf);
	//   ST7789_DrawImage(16,52,240,135,displayBuf);
	}
  */

  while (true) 
  {//横向
    gpio_put(PIN_LED, !gpio_get(PIN_LED));
    arducam_capture_frame(&config);

    uint16_t index = 0;
    for (int y = 0; y < 240; y++) {
      for (int x = 0; x < 135; x++) {
        //uint8_t c = image_buf[(2+320-2*y)*324+(2+40+2*x)]; 原数据
        uint8_t c = image_buf[(y)*324+(x)];
        uint16_t imageRGB   = ST7735_COLOR565(c, c, c);
        displayBuf[index++] = (uint8_t)(imageRGB >> 8) & 0xFF;
        displayBuf[index++] = (uint8_t)(imageRGB)&0xFF;
      }
    }
    ST7735_DrawImage(52, 40, 135, 240, displayBuf);
    if(gpio_get(BUTTON_PIN1) == 0)
    {
      play_melody(&noteTimer, HappyBirday,140);
    }
    else if(gpio_get(BUTTON_PIN2) == 0)
    {
      play_melody(&noteTimer, HarryPotter,140);
    }
    
  }

}

int main() 
{
  int loops=20;
  stdio_init_all();
  /*music*/
  gpio_set_function(PWM_PIN, GPIO_FUNC_PWM);
  // Find out which PWM slice is connected to GPIO 0 (it's slice 0)
  uint  slice_num = pwm_gpio_to_slice_num(PWM_PIN);
  noteTimer.slice_num= slice_num;
  /*music*/
  //KEY
  gpio_init(LED_PIN1);
  gpio_set_dir(LED_PIN1, GPIO_OUT);
  gpio_init(BUTTON_PIN1);
  gpio_set_dir(BUTTON_PIN1, GPIO_IN);
  gpio_set_pulls(BUTTON_PIN1, true, false);  // 设置按键为上拉模式
  gpio_init(LED_PIN2);
  gpio_set_dir(LED_PIN2, GPIO_OUT);
  gpio_init(BUTTON_PIN2);
  gpio_set_dir(BUTTON_PIN2, GPIO_IN);
  gpio_set_pulls(BUTTON_PIN2, true, false);  // 设置按键为上拉模式

      //run music
  play_melody(&noteTimer, Power_On_Prompt,140);
  // wait until is done
  //while(!noteTimer.Done);


  while (!tud_cdc_connected()) { sleep_ms(100); if (--loops==0) break;}

  printf("tud_cdc_connected(%d)\n", tud_cdc_connected()?1:0);

  vreg_set_voltage(VREG_VOLTAGE_1_30);
  sleep_ms(1000);
  set_sys_clock_khz(250000, true);

  multicore_launch_core1(core1_entry);

  uint32_t g = multicore_fifo_pop_blocking();

  if (g != FLAG_VALUE)
    printf("Hmm, that's not right on core 0!\n");
  else {
    multicore_fifo_push_blocking(FLAG_VALUE);
    printf("It's all gone well on core 0!\n");
  }

  while (1)
  {
    tight_loop_contents();
	  //playsong(song, sizeof(song) / sizeof(song[0]));
    /*
    if (gpio_get(BUTTON_PIN1) == 0) {
        // 按键被按下
        printf("Button pressed! Turning on the LED.\n");
        gpio_put(LED_PIN1, 1);  // 点亮LED
        playsong(song, sizeof(song) / sizeof(song[0]));
    } else {
        // 按键未被按下
        printf("Button not pressed. Turning off the LED.\n");
        gpio_put(LED_PIN1, 0);  // 关闭LED
    }
    if (gpio_get(BUTTON_PIN2) == 0) {
        // 按键被按下
        printf("Button pressed! Turning on the LED.\n");
        playsong(song, sizeof(song) / sizeof(song[0]));
        gpio_put(LED_PIN2, 1);  // 点亮LED
    } else {
        // 按键未被按下
        printf("Button not pressed. Turning off the LED.\n");
        gpio_put(LED_PIN2, 0);  // 关闭LED
    }
    */
  }
}
